require ("_prolog")
require ("_opencv")
function main()
  if (not OcvEnabled) then
     error(errOcvDisabled)  
     return     
  end         
  -- target images must be loaded  
  if (TargetImage.Plane == nil) then
      error(errTargetEmpty)
      return
  end       
  ocv_InitFFI() --initialize FFIs
  local inpImage = lips_CreateImage("rgb24", SourceImage.Width, SourceImage.Height)
  if SourceMask.Plane == nil then
     lips_CopyImage(SourceImage, inpImage)      
  else
     lips_Convert_grayscale2rgb(SourceMask, inpImage)
  end
  -- set OpenCV src image
  ocv_SetImage("source", inpImage)
  ocv_SetImage("target", SourceImage)  -- just to satisfy src/tgt existance  
  ocv_FlipSource()  -- flipping a copy of source image 
  local params, min_points
  if SilentParams ~= nil then
     local pos =  SilentParams:find(",") 
     local tok1 = SilentParams:sub(1, pos-1)
     local tok2 = SilentParams:sub(pos+1)
     min_points = tonumber(tok1)
     params = "0, "..tok2
  else
     min_points = 2
     params = "0, 1, 1" 
  end
  ocv_Process("FindContours", params)  
  local contoursNum = ocv_GetContoursNum()
  local contour
  -- count number of contours of interest
  local outNumContours = 0
  for idx = 0, contoursNum-1 do
      contour = ocv_GetContour(idx)
      if (contour ~= nil) then
         if (contour.numPts > min_points) then
            outNumContours = outNumContours + 1  
         end 
      end
  end
  lips_SendCommand("NumContours", tostring(outNumContours))  
  --- output each contour
  for idx = 0, contoursNum-1 do
      contour = ocv_GetContour(idx)
      if (contour ~= nil) then
         if (contour.numPts > min_points) then
            lips_ExportContour() 
         end 
      end
  end
  ocv_Process("ReleaseContours")
end
