require ("_prolog")
require ("_opencv")
function RequireParams()  
  if ExeWin32 then
      lips_RequireParams("model = combo([ESPCN x2, ESPCN x3, ESPCN x4, FSRCNN x2, FSRCNN x3, FSRCNN x4, LapSRN x2] 0);")
  else
      lips_RequireParams("model = combo([ESPCN x2, ESPCN x3, ESPCN x4, FSRCNN x2, FSRCNN x3, FSRCNN x4, LapSRN x2, LapSRN x4, LapSRN x8, EDSR x2, EDSR x3, EDSR x4] 0);")
  end  
end   
function main()
  if (not OcvEnabled) then
     lips_SendMessage(errOcvDisabled)  
     return     
  end       
  local model_dnld = "https://github.com/opencv/opencv_contrib/tree/master/modules/dnn_superres"
  local model_name, algorithm, scale
  if model == 0 then
     model_name = ExePath.."data/superres/ESPCN_x2.pb"     
     algorithm = 0
     scale = 2     
  elseif  model == 1 then
     model_name = ExePath.."data/superres/ESPCN_x3.pb"     
     algorithm = 0
     scale = 3
  elseif  model == 2 then
     model_name = ExePath.."data/superres/ESPCN_x4.pb"     
     algorithm = 0
     scale = 4
  elseif  model == 3 then        
     model_name = ExePath.."data/superres/FSRCNN_x2.pb"         
     algorithm = 1
     scale = 2          
  elseif model == 4 then
     model_name = ExePath.."data/superres/FSRCNN_x3.pb"         
     algorithm = 1
     scale = 3          
  elseif model == 5 then
     model_name = ExePath.."data/superres/FSRCNN_x4.pb"         
     algorithm = 1
     scale = 4          
  elseif model == 6 then   
     model_name = ExePath.."data/superres/LapSRN_x2.pb"               
     algorithm = 2
     scale = 2          
  elseif model == 7 then   
     model_name = ExePath.."data/superres/LapSRN_x4.pb"               
     algorithm = 2
     scale = 4          
  elseif model == 8 then   
     model_name = ExePath.."data/superres/LapSRN_x8.pb"               
     algorithm = 2
     scale = 8             
  elseif model == 9 then   
     model_name = ExePath.."data/superres/EDSR_x2.pb"      
     algorithm = 3
     scale = 2                  
  elseif model == 10 then   
     model_name = ExePath.."data/superres/EDSR_x3.pb"      
     algorithm = 3
     scale = 3                    
  else
     model_name = ExePath.."data/superres/EDSR_x4.pb"      
     algorithm = 3
     scale = 4                         
  end
  if (not lips_Check("FileExists", model_name)) then
    lips_SendMessage("Error: Model file missing.")      
    lips_HostDialog('Model file missing. Download model file from:<br><A href="'..model_dnld..'">Super Resolution using Convolutional Neural Networks</A><br> and place it in <b>data/superres/</b> folder.', 2)    
    return
  end    
  local inpImage = lips_CreateImage("rgb24", SourceImage.Width, SourceImage.Height)    
  lips_CopyImage(SourceImage, inpImage)    
  -- set OpenCV src image
  ocv_SetImage("source", inpImage)
  ocv_SetImage("target", SourceImage)  -- just to satisfy src/tgt existance  
  ocv_FlipSource()  -- flipping a copy of source image  
  ocv_Process("DnnSuperResolution", model_name, algorithm, scale)  
  local id = ocv_GetLastImageId()  
  local outImage = lips_GetImageFromId(id)
  if (outImage ~= nil) then   
     lips_ExportImage(outImage, "superres result")   
  end 
end

